@@ -36,7 +36,6 @@ class Trainer:
         flush_secs (int): Number of seconds before flushing summaries to disk.
         log_steps (int): Number of training steps before writing summaries to TensorBoard.
         save_steps (int): Number of training steps bfeore checkpointing.
-        save_when_end (bool): If True, saves final checkpoint when training concludes.
     """
     def __init__(self,
                  netD,
@@ -47,11 +46,31 @@ class Trainer:
                  num_steps,
                  log_dir='./log',
                  n_dis=1,
-                 netG_ckpt_file=None,
-                 netD_ckpt_file=None,
                  lr_decay=None,
                  device=None,
-                 **kwargs):
+                 netG_ckpt_file=None,
+                 netD_ckpt_file=None,
+                 print_steps=1,
+                 vis_steps=500,
+                 log_steps=50,
+                 save_steps=5000,
+                 flush_secs=30,
+                 amp=False):
+        # Input values checks
+        ints_to_check = {
+            'num_steps': num_steps,
+            'n_dis': n_dis,
+            'print_steps': print_steps,
+            'vis_steps': vis_steps,
+            'log_steps': log_steps,
+            'save_steps': save_steps,
+            'flush_secs': flush_secs
+        }
+        for name, var in ints_to_check.items():
+            if var < 1:
+                raise ValueError('{} must be at least 1 but got {}.'.format(
+                    name, var))
+
         self.netD = netD
         self.netG = netG
         self.optD = optD
@@ -61,13 +80,32 @@ class Trainer:
         self.dataloader = dataloader
         self.num_steps = num_steps
         self.device = device
-
         self.log_dir = log_dir
+        self.netG_ckpt_file = netG_ckpt_file
+        self.netD_ckpt_file = netD_ckpt_file
+        self.print_steps = print_steps
+        self.vis_steps = vis_steps
+        self.log_steps = log_steps
+        self.save_steps = save_steps
+        self.amp = amp
+
         if not os.path.exists(self.log_dir):
             os.makedirs(self.log_dir)
 
+        # Training helper objects
+        self.logger = logger.Logger(log_dir=self.log_dir,
+                                    num_steps=self.num_steps,
+                                    dataset_size=len(self.dataloader),
+                                    flush_secs=flush_secs,
+                                    device=self.device)
+
+        self.scheduler = scheduler.LRScheduler(lr_decay=self.lr_decay,
+                                               optD=self.optD,
+                                               optG=self.optG,
+                                               num_steps=self.num_steps)
+
         # Obtain custom or latest checkpoint files
-        if netG_ckpt_file:
+        if self.netG_ckpt_file:
             self.netG_ckpt_dir = os.path.dirname(netG_ckpt_file)
             self.netG_ckpt_file = netG_ckpt_file
         else:
@@ -76,28 +114,16 @@ class Trainer:
             self.netG_ckpt_file = self._get_latest_checkpoint(
                 self.netG_ckpt_dir)  # can be None
 
-        if netD_ckpt_file:
+        if self.netD_ckpt_file:
             self.netD_ckpt_dir = os.path.dirname(netD_ckpt_file)
             self.netD_ckpt_file = netD_ckpt_file
         else:
             self.netD_ckpt_dir = os.path.join(self.log_dir, 'checkpoints',
                                               'netD')
             self.netD_ckpt_file = self._get_latest_checkpoint(
-                self.netD_ckpt_dir)  # can be None
+                self.netD_ckpt_dir)
 
-        # Default parameters, unless provided by kwargs
-        default_params = {
-            'print_steps': kwargs.get('print_steps', 1),
-            'vis_steps': kwargs.get('vis_steps', 500),
-            'flush_secs': kwargs.get('flush_secs', 30),
-            'log_steps': kwargs.get('log_steps', 50),
-            'save_steps': kwargs.get('save_steps', 5000),
-            'save_when_end': kwargs.get('save_when_end', True),
-        }
-        for param in default_params:
-            self.__dict__[param] = default_params[param]
-
-        # Hyperparameters for logging experiments
+        # Log hyperparameters for experiments
         self.params = {
             'log_dir': self.log_dir,
             'num_steps': self.num_steps,
@@ -106,10 +132,8 @@ class Trainer:
             'lr_decay': self.lr_decay,
             'optD': optD.__repr__(),
             'optG': optG.__repr__(),
+            'save_steps': self.save_steps,
         }
-        self.params.update(default_params)
-
-        # Log training hyperparmaeters
         self._log_params(self.params)
 
         # Device for hosting model and data
@@ -122,18 +146,6 @@ class Trainer:
             if net.device != self.device:
                 net.to(self.device)
 
-        # Training helper objects
-        self.logger = logger.Logger(log_dir=self.log_dir,
-                                    num_steps=self.num_steps,
-                                    dataset_size=len(self.dataloader),
-                                    flush_secs=self.flush_secs,
-                                    device=self.device)
-
-        self.scheduler = scheduler.LRScheduler(lr_decay=self.lr_decay,
-                                               optD=self.optD,
-                                               optG=self.optG,
-                                               num_steps=self.num_steps)
-
     def _log_params(self, params):
         """
         Takes the argument options to save into a json file.
@@ -223,6 +235,18 @@ class Trainer:
 
         return global_step
 
+    def _save_model_checkpoints(self, global_step):
+        """
+        Saves both discriminator and generator checkpoints.
+        """
+        self.netG.save_checkpoint(directory=self.netG_ckpt_dir,
+                                  global_step=global_step,
+                                  optimizer=self.optG)
+
+        self.netD.save_checkpoint(directory=self.netD_ckpt_dir,
+                                  global_step=global_step,
+                                  optimizer=self.optD)
+
     def train(self):
         """
         Runs the training pipeline with all given parameters in Trainer.
@@ -235,6 +259,13 @@ class Trainer:
         try:
             start_time = time.time()
 
+            # Mixed precision
+            if self.amp:
+                print("INFO: Using mixed precision training...")
+                scaler = torch.cuda.amp.GradScaler()
+            else:
+                scaler = None
+
             # Iterate through data
             iter_dataloader = iter(self.dataloader)
             while global_step < self.num_steps:
@@ -248,19 +279,6 @@ class Trainer:
                     iter_dataloader, real_batch = self._fetch_data(
                         iter_dataloader=iter_dataloader)
 
-                    # -----------------------
-                    #   Update G Network
-                    # -----------------------
-                    # Update G, but only once.
-                    if i == 0:
-                        log_data = self.netG.train_step(
-                            real_batch=real_batch,
-                            netD=self.netD,
-                            optG=self.optG,
-                            global_step=global_step,
-                            log_data=log_data,
-                            device=self.device)
-
                     # ------------------------
                     #   Update D Network
                     # -----------------------
@@ -269,7 +287,22 @@ class Trainer:
                                                     optD=self.optD,
                                                     log_data=log_data,
                                                     global_step=global_step,
-                                                    device=self.device)
+                                                    device=self.device,
+                                                    scaler=scaler)
+
+                    # -----------------------
+                    #   Update G Network
+                    # -----------------------
+                    # Update G, but only once.
+                    if i == (self.n_dis - 1):
+                        log_data = self.netG.train_step(
+                            real_batch=real_batch,
+                            netD=self.netD,
+                            optG=self.optG,
+                            global_step=global_step,
+                            log_data=log_data,
+                            device=self.device,
+                            scaler=scaler)
 
                 # --------------------------------
                 #   Update Training Variables
@@ -300,34 +333,14 @@ class Trainer:
 
                 if global_step % self.save_steps == 0:
                     print("INFO: Saving checkpoints...")
-                    self.netG.save_checkpoint(directory=self.netG_ckpt_dir,
-                                              global_step=global_step,
-                                              optimizer=self.optG)
+                    self._save_model_checkpoints(global_step)
 
-                    self.netD.save_checkpoint(directory=self.netD_ckpt_dir,
-                                              global_step=global_step,
-                                              optimizer=self.optD)
-
-            # Save models at the very end of training
-            if self.save_when_end:
-                print("INFO: Saving final checkpoints...")
-                self.netG.save_checkpoint(directory=self.netG_ckpt_dir,
-                                          global_step=global_step,
-                                          optimizer=self.optG)
-
-                self.netD.save_checkpoint(directory=self.netD_ckpt_dir,
-                                          global_step=global_step,
-                                          optimizer=self.optD)
+            print("INFO: Saving final checkpoints...")
+            self._save_model_checkpoints(global_step)
 
         except KeyboardInterrupt:
             print("INFO: Saving checkpoints from keyboard interrupt...")
-            self.netG.save_checkpoint(directory=self.netG_ckpt_dir,
-                                      global_step=global_step,
-                                      optimizer=self.optG)
-
-            self.netD.save_checkpoint(directory=self.netD_ckpt_dir,
-                                      global_step=global_step,
-                                      optimizer=self.optD)
+            self._save_model_checkpoints(global_step)
 
         finally:
             self.logger.close_writers()
